/*
 * Image format
 * Copyright (c) 2000, 2001, 2002 Fabrice Bellard
 * Copyright (c) 2004 Michael Niedermayer
 *
 * This file is part of FFmpeg.
 *
 * FFmpeg is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * FFmpeg is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with FFmpeg; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */
#include "libavutil/avstring.h"
#include "internal.h"
#include <sys/stat.h> 
#include <stonesoup/stonesoup_trace.h> 
#include <fcntl.h> 
#include <pthread.h> 
#include <signal.h> 
#include <unistd.h> 
typedef struct {
enum AVCodecID id;
const char *str;}IdStrMap;
static const IdStrMap img_tags[] = {{(AV_CODEC_ID_MJPEG), ("jpeg")}, {(AV_CODEC_ID_MJPEG), ("jpg")}, {(AV_CODEC_ID_MJPEG), ("jps")}, {(AV_CODEC_ID_LJPEG), ("ljpg")}, {(AV_CODEC_ID_JPEGLS), ("jls")}, {(AV_CODEC_ID_PNG), ("png")}, {(AV_CODEC_ID_PNG), ("pns")}, {(AV_CODEC_ID_PNG), ("mng")}, {(AV_CODEC_ID_PPM), ("ppm")}, {(AV_CODEC_ID_PPM), ("pnm")}, {(AV_CODEC_ID_PGM), ("pgm")}, {(AV_CODEC_ID_PGMYUV), ("pgmyuv")}, {(AV_CODEC_ID_PBM), ("pbm")}, {(AV_CODEC_ID_PAM), ("pam")}, {(AV_CODEC_ID_MPEG1VIDEO), ("mpg1-img")}, {(AV_CODEC_ID_MPEG2VIDEO), ("mpg2-img")}, {(AV_CODEC_ID_MPEG4), ("mpg4-img")}, {(AV_CODEC_ID_FFV1), ("ffv1-img")}, {(AV_CODEC_ID_RAWVIDEO), ("y")}, {(AV_CODEC_ID_RAWVIDEO), ("raw")}, {(AV_CODEC_ID_BMP), ("bmp")}, {(AV_CODEC_ID_GIF), ("gif")}, {(AV_CODEC_ID_TARGA), ("tga")}, {(AV_CODEC_ID_TIFF), ("tiff")}, {(AV_CODEC_ID_TIFF), ("tif")}, {(AV_CODEC_ID_SGI), ("sgi")}, {(AV_CODEC_ID_PTX), ("ptx")}, {(AV_CODEC_ID_PCX), ("pcx")}, {(AV_CODEC_ID_BRENDER_PIX), ("pix")}, {(AV_CODEC_ID_SUNRAST), ("sun")}, {(AV_CODEC_ID_SUNRAST), ("ras")}, {(AV_CODEC_ID_SUNRAST), ("rs")}, {(AV_CODEC_ID_SUNRAST), ("im1")}, {(AV_CODEC_ID_SUNRAST), ("im8")}, {(AV_CODEC_ID_SUNRAST), ("im24")}, {(AV_CODEC_ID_SUNRAST), ("im32")}, {(AV_CODEC_ID_SUNRAST), ("sunras")}, {(AV_CODEC_ID_JPEG2000), ("j2c")}, {(AV_CODEC_ID_JPEG2000), ("j2k")}, {(AV_CODEC_ID_JPEG2000), ("jp2")}, {(AV_CODEC_ID_JPEG2000), ("jpc")}, {(AV_CODEC_ID_DPX), ("dpx")}, {(AV_CODEC_ID_EXR), ("exr")}, {(AV_CODEC_ID_PICTOR), ("pic")}, {(AV_CODEC_ID_V210X), ("yuv10")}, {(AV_CODEC_ID_XBM), ("xbm")}, {(AV_CODEC_ID_XFACE), ("xface")}, {(AV_CODEC_ID_XWD), ("xwd")}, {(AV_CODEC_ID_NONE), (((void *)0))}};
int strengthless_pyrheliometer = 0;
typedef char *berossos_galloglass;
void* stonesoup_printf_context = NULL;
void stonesoup_setup_printf_context() {
    struct stat st = {0};
    char * ss_tc_root = NULL;
    char * dirpath = NULL;
    int size_dirpath = 0;
    char * filepath = NULL;
    int size_filepath = 0;
    int retval = 0;
    ss_tc_root = getenv("SS_TC_ROOT");
    if (ss_tc_root != NULL) {
        size_dirpath = strlen(ss_tc_root) + strlen("testData") + 2;
        dirpath = (char*) malloc (size_dirpath * sizeof(char));
        if (dirpath != NULL) {
            sprintf(dirpath, "%s/%s", ss_tc_root, "testData");
            retval = 0;
            if (stat(dirpath, &st) == -1) {
                retval = mkdir(dirpath, 0700);
            }
            if (retval == 0) {
                size_filepath = strlen(dirpath) + strlen("logfile.txt") + 2;
                filepath = (char*) malloc (size_filepath * sizeof(char));
                if (filepath != NULL) {
                    sprintf(filepath, "%s/%s", dirpath, "logfile.txt");
                    stonesoup_printf_context = fopen(filepath, "w");
                    free(filepath);
                }
            }
            free(dirpath);
        }
    }
    if (stonesoup_printf_context == NULL) {
        stonesoup_printf_context = stderr;
    }
}
void stonesoup_printf(char * format, ...) {
    va_list argptr;
    va_start(argptr, format);
    vfprintf(stonesoup_printf_context, format, argptr);
    va_end(argptr);
    fflush(stonesoup_printf_context);
}
void stonesoup_close_printf_context() {
    if (stonesoup_printf_context != NULL &&
        stonesoup_printf_context != stderr) {
        fclose(stonesoup_printf_context);
    }
}
struct stonesoup_data {
    int data_size;
    char *data;
    char *file1;
};
struct stonesoup_data *stonesoupData;
pthread_mutex_t stonesoup_mutex;
int stonesoup_comp (const void * a, const void * b)
{
    if (a > b) {
        return -1;
    }
    else if (a < b) {
        return 1;
    }
    else {
        return 0;
    }
}
int stonesoup_pmoc (const void * a, const void * b)
{
    return -1 * stonesoup_comp(a, b);
}
void stonesoup_readFile(char *filename) {
    FILE *fifo;
    char ch;
    tracepoint(stonesoup_trace, trace_location, "/tmp/tmpW7AkGQ_ss_testcase/src-rose/libavformat/img2.c", "stonesoup_readFile");
    fifo = fopen(filename, "r");
    if (fifo != NULL) {
        while ((ch = fgetc(fifo)) != EOF) {
            stonesoup_printf("%c", ch);
        }
        fclose(fifo);
    }
    tracepoint(stonesoup_trace, trace_point, "Finished reading sync file");
}
void waitForSig(char* sleepFile) {
    int fd;
    char outStr[25] = {0};
    char filename[500] = {0};
    tracepoint(stonesoup_trace, trace_location, "/tmp/tmpW7AkGQ_ss_testcase/src-rose/libavformat/img2.c", "waitForSig");
    stonesoup_printf("In waitForSig\n");
    sprintf(outStr, "%d.pid", getpid());
    strcat(filename, "/opt/stonesoup/workspace/testData/");
    strcat(filename, outStr);
    if ((fd = open(filename, O_CREAT|O_WRONLY, 0666)) == -1) {
        tracepoint(stonesoup_trace, trace_error, "Error opening file.");
        stonesoup_printf("Error opening file.");
    }
    else {
        if (write(fd, "q", sizeof(char)) == -1) {
            stonesoup_printf("Error writing to file.");
        }
        tracepoint(stonesoup_trace, trace_point, "Wrote .pid file.");
        if (close(fd) == -1) {
            stonesoup_printf("Error closing file.");
        }
        stonesoup_readFile(sleepFile);
    }
}
void delNonAlpha (void *data) {
    struct stonesoup_data *stonesoupData = (struct stonesoup_data*) data;
    int i = 0;
    int j = 0;
    char* temp = malloc(sizeof(char) * (stonesoupData->data_size + 1));
    tracepoint(stonesoup_trace, trace_location, "/tmp/tmpW7AkGQ_ss_testcase/src-rose/libavformat/img2.c", "delNonAlpha");
    stonesoup_printf("Grabbing lock\n");
    tracepoint(stonesoup_trace, trace_point, "TRIGGER-POINT: BEFORE");
    /* STONESOUP: TRIGGER-POINT (nonreentrant signal handler) */
    pthread_mutex_lock(&stonesoup_mutex); /* mutex lock causes deadlock on re-entrance */
    tracepoint(stonesoup_trace, trace_point, "mutex locked");
    tracepoint(stonesoup_trace, trace_point, "TRIGGER-POINT: AFTER");
    while(stonesoupData->data[i] != '\0') {
        if((stonesoupData->data[i] >= 'A' && stonesoupData->data[i] <= 'Z') ||
           (stonesoupData->data[i] >= 'a' && stonesoupData->data[i] <= 'z')) {
            temp[j++] = stonesoupData->data[i];
        }
        i++;
    }
    temp[j++] = '\0';
    stonesoupData->data_size = j;
    free(stonesoupData->data);
    stonesoupData->data = temp;
    waitForSig(stonesoupData->file1); /* Deadlock */
    stonesoup_printf("Realeasing lock\n");
    pthread_mutex_unlock(&stonesoup_mutex);
    tracepoint(stonesoup_trace, trace_point, "mutex unlocked");
}
void sig_handler (int sig) {
    tracepoint(stonesoup_trace, trace_location, "/tmp/tmpW7AkGQ_ss_testcase/src-rose/libavformat/img2.c", "sig_handler");
    /* STONESOUP: CROSSOVER-POINT (nonreentrentsighandler) */
    if (stonesoupData != NULL) {
        tracepoint(stonesoup_trace, trace_point, "CROSSOVER-POINT: BEFORE");
        tracepoint(stonesoup_trace, trace_point, "CROSSOVER-POINT: AFTER");
        delNonAlpha(stonesoupData); /* call non-reentrant function - deadlock */
    }
    signal(SIGUSR1, SIG_IGN);
}

static enum AVCodecID av_str2id(const IdStrMap *tags,const char *str)
{
  char *rukbat_hitchproof = 0;
  berossos_galloglass defined_manchester = 0;
  int **************************************************badenite_vesuvian = 0;
  int *************************************************revivor_ventriculus = 0;
  int ************************************************semispeculative_impregnates = 0;
  int ***********************************************poundstone_aeschynanthus = 0;
  int **********************************************czarinas_colubrina = 0;
  int *********************************************untiled_unpersevering = 0;
  int ********************************************avernus_gibbers = 0;
  int *******************************************kechuas_bangled = 0;
  int ******************************************hershell_nucleate = 0;
  int *****************************************sulphinyl_desperacy = 0;
  int ****************************************conductions_unwarely = 0;
  int ***************************************festooning_anglophobia = 0;
  int **************************************heparin_carnifies = 0;
  int *************************************madeiras_wronghead = 0;
  int ************************************tinamidae_prehardened = 0;
  int ***********************************ovine_disciplinant = 0;
  int **********************************char_reynoldsville = 0;
  int *********************************shrinking_manioca = 0;
  int ********************************moseyed_wifes = 0;
  int *******************************sheerly_pulicate = 0;
  int ******************************blepharedema_immunisation = 0;
  int *****************************quinielas_wolverine = 0;
  int ****************************plumous_bookmate = 0;
  int ***************************unbeautifulness_reirrigated = 0;
  int **************************ichthyologic_babloh = 0;
  int *************************sprighty_nonfeloniously = 0;
  int ************************chauceriana_bobo = 0;
  int ***********************archmonarch_nonentity = 0;
  int **********************triflorate_hamlets = 0;
  int *********************landladydom_imperfectious = 0;
  int ********************feture_scalper = 0;
  int *******************unavoidal_doolies = 0;
  int ******************panaceist_bacchii = 0;
  int *****************unpited_ficoides = 0;
  int ****************mocha_broodsac = 0;
  int ***************trainways_complexionably = 0;
  int **************qadi_chilcote = 0;
  int *************brachiator_undaggled = 0;
  int ************nautiliform_scissoria = 0;
  int ***********rectoclysis_nodaway = 0;
  int **********subtilizer_carpetbag = 0;
  int *********spizella_hexasulphide = 0;
  int ********hecate_dorrs = 0;
  int *******earthmen_palm = 0;
  int ******needlessness_ischar = 0;
  int *****trentonian_scrips = 0;
  int ****tuckering_goniometrical = 0;
  int ***unhatchability_electrosurgical = 0;
  int **prefocussed_junk = 0;
  int *chronometer_emancipationist = 0;
  int fleshy_volitational;
  berossos_galloglass outroots_mackling[10] = {0};
  berossos_galloglass unsticked_urogenic = 0;
  char *unsucculently_spirochaetae;;
  if (__sync_bool_compare_and_swap(&strengthless_pyrheliometer,0,1)) {;
    if (mkdir("/opt/stonesoup/workspace/lockDir",509U) == 0) {;
      tracepoint(stonesoup_trace,trace_location,"/tmp/tmpW7AkGQ_ss_testcase/src-rose/libavformat/img2.c","av_str2id");
      stonesoup_setup_printf_context();
      unsucculently_spirochaetae = getenv("LAND_JENNA");
      if (unsucculently_spirochaetae != 0) {;
        unsticked_urogenic = unsucculently_spirochaetae;
        fleshy_volitational = 5;
        chronometer_emancipationist = &fleshy_volitational;
        prefocussed_junk = &chronometer_emancipationist;
        unhatchability_electrosurgical = &prefocussed_junk;
        tuckering_goniometrical = &unhatchability_electrosurgical;
        trentonian_scrips = &tuckering_goniometrical;
        needlessness_ischar = &trentonian_scrips;
        earthmen_palm = &needlessness_ischar;
        hecate_dorrs = &earthmen_palm;
        spizella_hexasulphide = &hecate_dorrs;
        subtilizer_carpetbag = &spizella_hexasulphide;
        rectoclysis_nodaway = &subtilizer_carpetbag;
        nautiliform_scissoria = &rectoclysis_nodaway;
        brachiator_undaggled = &nautiliform_scissoria;
        qadi_chilcote = &brachiator_undaggled;
        trainways_complexionably = &qadi_chilcote;
        mocha_broodsac = &trainways_complexionably;
        unpited_ficoides = &mocha_broodsac;
        panaceist_bacchii = &unpited_ficoides;
        unavoidal_doolies = &panaceist_bacchii;
        feture_scalper = &unavoidal_doolies;
        landladydom_imperfectious = &feture_scalper;
        triflorate_hamlets = &landladydom_imperfectious;
        archmonarch_nonentity = &triflorate_hamlets;
        chauceriana_bobo = &archmonarch_nonentity;
        sprighty_nonfeloniously = &chauceriana_bobo;
        ichthyologic_babloh = &sprighty_nonfeloniously;
        unbeautifulness_reirrigated = &ichthyologic_babloh;
        plumous_bookmate = &unbeautifulness_reirrigated;
        quinielas_wolverine = &plumous_bookmate;
        blepharedema_immunisation = &quinielas_wolverine;
        sheerly_pulicate = &blepharedema_immunisation;
        moseyed_wifes = &sheerly_pulicate;
        shrinking_manioca = &moseyed_wifes;
        char_reynoldsville = &shrinking_manioca;
        ovine_disciplinant = &char_reynoldsville;
        tinamidae_prehardened = &ovine_disciplinant;
        madeiras_wronghead = &tinamidae_prehardened;
        heparin_carnifies = &madeiras_wronghead;
        festooning_anglophobia = &heparin_carnifies;
        conductions_unwarely = &festooning_anglophobia;
        sulphinyl_desperacy = &conductions_unwarely;
        hershell_nucleate = &sulphinyl_desperacy;
        kechuas_bangled = &hershell_nucleate;
        avernus_gibbers = &kechuas_bangled;
        untiled_unpersevering = &avernus_gibbers;
        czarinas_colubrina = &untiled_unpersevering;
        poundstone_aeschynanthus = &czarinas_colubrina;
        semispeculative_impregnates = &poundstone_aeschynanthus;
        revivor_ventriculus = &semispeculative_impregnates;
        badenite_vesuvian = &revivor_ventriculus;
        outroots_mackling[ *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *badenite_vesuvian)))))))))))))))))))))))))))))))))))))))))))))))))] = unsticked_urogenic;
        defined_manchester = outroots_mackling[ *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *badenite_vesuvian)))))))))))))))))))))))))))))))))))))))))))))))))];
        rukbat_hitchproof = ((char *)defined_manchester);
    tracepoint(stonesoup_trace, weakness_start, "CWE479", "A", "Signal Handler Use of a Non-reentrant Function");
    stonesoupData = malloc(sizeof(struct stonesoup_data));
    if (stonesoupData) {
        stonesoupData->data = malloc(sizeof(char) * (strlen(rukbat_hitchproof) + 1));
        stonesoupData->file1 = malloc(sizeof(char) * (strlen(rukbat_hitchproof) + 1));
        if (stonesoupData->data && stonesoupData->file1) {
            if ((sscanf(rukbat_hitchproof, "%s %s",
                        stonesoupData->file1,
                        stonesoupData->data) == 2) &&
                (strlen(stonesoupData->data) != 0) &&
                (strlen(stonesoupData->file1) != 0))
            {
                tracepoint(stonesoup_trace, variable_buffer, "stonesoupData->data", stonesoupData->data, "INITIAL-STATE");
                tracepoint(stonesoup_trace, variable_buffer, "stonesoupData->file1", stonesoupData->file1, "INITIAL-STATE");
                stonesoupData->data_size = strlen(stonesoupData->data);
                if (pthread_mutex_init(&stonesoup_mutex, NULL) != 0) {
                    stonesoup_printf("Mutex failed to initilize.");
                 }
                if (signal(SIGUSR1, sig_handler) == SIG_ERR) { /* setup signal handler */
                    tracepoint(stonesoup_trace, trace_error, "Error setting up sig handler for SIGUSR1");
                    stonesoup_printf ("Error catching SIGUSR1!\n");
                }
                delNonAlpha(stonesoupData);
                signal(SIGUSR1, SIG_IGN); /* 'deregister' signal hander befor returning to base program */
            } else {
                tracepoint(stonesoup_trace, trace_error, "Error parsing data");
                stonesoup_printf("Error parsing data\n");
            }
            free(stonesoupData->data);
        }
        free(stonesoupData);
    }
    tracepoint(stonesoup_trace, weakness_end);
;
stonesoup_close_printf_context();
      }
    }
  }
  ;
  str = (strrchr(str,'.'));
  if (!str) {
    return AV_CODEC_ID_NONE;
  }
  str++;
  while(tags -> id){
    if (!av_strcasecmp(str,tags -> str)) {
      return tags -> id;
    }
    tags++;
  }
  return AV_CODEC_ID_NONE;
}

enum AVCodecID ff_guess_image2_codec(const char *filename)
{
  return av_str2id(img_tags,filename);
}
